% BALM v 0.1
% Authors: A. Del Bue, J. Xavier, L. Agapito, M. Paladini
%
% These functions implement the BALM algorithm presented in:  A. Del Bue,
% J. Xavier, L. Agapito, and M. Paladini, "Bilinear Factorization via
% Augmented Lagrange Multipliers (BALM)" ECCV 2010. We suggest the reading
% of the article to understand specific details of the implementation here
% provided. The code is delivered under GPLv2 license.
%
% DESCRIPTION: the functions provide a core optimisation method for
% computing a bilinear decomposition of a given set of measurements Y i.e.
% we obtain the following factorisation Y = S M.
% Additionally, the user can enforce known manifold constraints on the
% matrix M and deal with missing entries in Y.
%
% You can also find two exemplar problems (more will appear soon):
% Photometric Stereo and Non-rigid Structure from motion. These problems
% can be run with the functions:
%
% demo_ps_missing.m
% demo_nrsfm_missing.m
%
% The data used reflects the experiments presented in the paper.
% The directory ./projectors/ contains the manifold projectors for the
% photometric stereo, non-rigid problems and some other variations (like
% different camera models for the SfM).
%
% If your specific application requires the creation of a custom manifold
% projector, please drop an email at alessio.delbue@iit. I will include
% your projector in future releases of the code.
%
% For any comments/suggestions you can contact me at alessio.delbue@iit.
%
% NOTES about the code:
% - The photometric stereo function requires an inpainting algorithm that
% it is not possible to include in this package (at the moment). Please
% check the instructions in balm_missing_photometric.m that provides
% pointers on where to download the code.
%
% - The function balm_your_bilinear_problem.m provides some instructions on
% how to create your custom optimisation. You may use this function as a
% starting point for adapting BALM to your problem.
%
% - Next version of the code will make available projectors and algorithms
% for solving 2D-3D registration problems.

% LICENCE
% 
% All the scripts contained in this distributions are Free Software
% under the GNU General Public License, version 2, June 1991
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2, June 1991 
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.