% Ref: A. Del Bue, J. Xavier, L. Agapito, and M. Paladini, "Bilinear
% Factorization via Augmented Lagrange Multipliers (BALM)" ECCV 2010.
%
%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; version 2, June 1991 
%
% USAGE: [WT,t] = balm_fact_register(W)
%
% This function register the 2D coordinates to the imge centroid at (0,0)
%
% INPUT: 
%
% W: the f2 x p matrix containing the 2D coordinates at each frame
%
% OUTPUT
%
% WT: the f2 x p matrix containing the registered 2D coordinates
% t:  the f2 x 1 vector storing the translation at each frame


function [WT,t] = balm_fact_register(W)
% Remove centroid from a measurement matrix W = p x 2f
%
%
[p,f2]=size(W);
t=zeros(f2,1);
WT=zeros(p,f2);
for i=1:f2
    %for every row, compute the mean and subtract it 
    t(i)=mean(W(:,i));
    WT(:,i) = W(:,i) - t(i);
end

end