% Ref: A. Del Bue, J. Xavier, L. Agapito, and M. Paladini, "Bilinear
% Factorization via Augmented Lagrange Multipliers (BALM)" ECCV 2010.
%
%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; version 2, June 1991 
%
% USAGE: [Yfill,M,S,t,err_1,timing] = balm_your_bilinear_problem(Y,D,@proj_handle,Kmax,Lmax);
%
% This function can be used as a template for solving your specific
% problem in bilinear form. In order to work, you need to provide the
% initialisation routine and the manifold project (passed as a function
% handle: @proj_handle)
%
%
% INPUT:
% Y: Measurement matrix storing each point trajectories at each column.
% the size of the matrix is m x n. Missing entries can be defined as a NaN
%
% D: A m x n matrix with the missing entries. "0" is an unknown entry, "1"
% is a known entry.
%
% proj_handle: a function handle to the function that does the single frame
% projection.
% 
% Kmax: maximum number of outer loop iterations (check Alg.1 in the paper)
%
% Lmax: maximum number of inner loop iterations (check Alg.2 in the paper)
%
% OUTPUT:
%
% Yfill: the new measurement matrix with the filled entries matrix with the
% filled entries. To notice that Yfill contains the original data for the 
% known entries of Y and the estimated missing data given by M,S and t 
% (i.e. Wfill = (M*S + repmat(t,1,size(S,2))).*not(D) + Y.*D)
%
% M: the estimated bilinear component with manifold constraints
%
% S: the estimated bilinear component
%
% t: the vector containing the translation
% 
% err_1: error at each iteration defined as "err_1(k)= norm(Yprev -
% Yk,'fro')/numel(Yk);"
%
% timing: optimisation elapsed time

function [Yfill,M,S,t,err_1,timing] = balm_your_bilinear_problem(Y,D,proj_handle,Kmax,Lmax)


%%%%%%%%%%%%%%%%%%% INITIALISATION %%%%%%%%%%%%%%%%%%%

% Fill missing entries using your algorithm of choice. This part should
% provide a Yinitial, Minitial, Sinitial, tinitial (if your problem 
% accounts for overall translation).

% Yinitial = ?;
% Minitial = ?;
% Sinitial = ?;
% tinitial = ?;

%%%%%%%%%%%%%%%%%%% CALL BALM %%%%%%%%%%%%%%%%%%%

req_cent = 1 % translation
req_cent = 0 % no translation

[S,M,t,timing,err_1] = balm_missing_bilinear_alter...
    (Yinitial,D,Sinitial,Minitial,req_cent,proj_handle,Kmax,Lmax); 

Dhat = ones(size(D))-D;
Yfill = S*M + repmat(t,1,size(S,2)); % find the imputed matrix
Yfill = Yfill.*Dhat + Y.*D;