%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; version 2, June 1991 
%
% USAGE: [basis,components]=pca_decomposition(DATA,r)
%
% Given a set of 3D trajectories in DATA, extract a set of r PCA basis.
%
%
% INPUT:
%
% DATA: the (f x 3p) matrix containing a set of f 3D trajectories where p
% is the number of 3D points.
%
% r: the number of PCA basis
%
% OUTPUT:
%
% basis: the pca baisis
% 
% components: the components given the extracted basis

function [basis,components]=pca_decomposition(DATA,r)
A=cov(register(DATA));
[ev,eg]=eig(A);
[u,ix]=sort(diag(eg),'descend');
basis = ev(:,ix(1:r));
components = basis'*DATA';
