% Ref: A. Del Bue, J. Xavier, L. Agapito, and M. Paladini, "Bilinear
% Factorization via Augmented Lagrange Multipliers (BALM)" ECCV 2010.
%
%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; version 2, June 1991 
%
% USAGE: plot_display_nrsfm(S, L, R, list_frames)
%
% The function plots a series of 3D shape for visual inspection. The shape
% are associated with their 3D rotation (R) when plotting.
%
% INPUT:
% 
% S: the (3*bases x P) shape matrix containing the 3D basis shapes
%
% L: the (bases x F) matrix containing the configurarion weights for the
% basis shapes
%
% R: the 3F x 3 collection of rotation matrices
%
% list_frames: a vector containing a list of 3D shapes to plot from the
% seguence

function plot_display_nrsfm(S, L, R, list_frames)

s = size(S);
k = s(1)/3; np = s(2); n_frame = size(L, 2); 
f=size(list_frames,2);


for m=1:f
    nframe = list_frames(m);
	lf = L(:, nframe); % Extract the weights for the frame m
	Si = zeros(3,np);
	for r = 0:k-1
        Si = Si + lf(r+1)*S(r*3 +1:r*3+3,:);    % Linear combination of k shapes 
	end
    Si = R(nframe*3 -2:nframe*3,:)*Si;
    Sf(f*3 +1:f*3+3,:)=Si;  % Store the shape for the frame f


        
    figure(nframe)  

    set(gcf,'Renderer','painters')
    set(gcf,'Color','white')
    plot3(Si(1,:), Si(2,:), Si(3,:), 'r.','MarkerSize',20)

%         grid on
    set(gcf,'PaperPositionMode','auto') 
    axis equal 
    axis manual


    
end

