% Ref: A. Del Bue, J. Xavier, L. Agapito, and M. Paladini, "Bilinear
% Factorization via Augmented Lagrange Multipliers (BALM)" ECCV 2010.
%
%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; version 2, June 1991 
%
% USAGE: Y = proj_photometric_4sph(X)
%
% This function projects a generic vector X of size 4 x 1 into the vector Y
% that satisfy the manifold constraints. This projection is enforcing the 
% normality constraints in the 1st order spherical harmonic. Check the BALM
% paper, Sec 6.1.
%
% INPUT
%
% X: the 4 x 1 affine vector 
%
% OUTPUT
%
% Y: the 4 x 1 vector with manifold constraints

function Y = proj_photometric_4sph(X)

if nargin < 1
    
    Y = 1; % this says on how many columns of M the projection must be done
    return
end

r = size(X,1);

if norm(X(2:4)) < 1e-12
    rho = X(1)/2;
    n = rand(3,1);
    n = n/norm(n);
    
else
    if X(1)>= 0
        rho = (X(1) + norm(X(2:4)))/2;
        n =  X(2:4)/norm(X(2:4));

    else

        rho = (X(1) - norm(X(2:4)))/2;
        n = -X(2:4)/norm(X(2:4));

    end
    
end

Y = rho*[1 ;n];