% Ref: A. Del Bue, J. Xavier, L. Agapito, and M. Paladini, "Bilinear
% Factorization via Augmented Lagrange Multipliers (BALM)" ECCV 2010.
%
%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; version 2, June 1991 
%
% USAGE: Rortho = proj_rigid_orthographic(R)
%
% This function projects a generic matrix X of size 3 x 2 into the matrix Y
% that satisfy the manifold constraints. This projection is enforcing the 
% ortographic camera constraints for Rigid SfM.
%
% INPUT
%
% R: the 3 x 2 affine matrix
%
% OUTPUT
%
% Rortho: the 3 x 2 matrix with manifold constraints (Stiefel)

function Rortho = proj_rigid_orthographic(R)

if nargin < 1    
    Rortho = 2; % this says on how many columns of M the projection must be done
    return
end

[U,S,V]=svd(R','econ');
Rortho=V*U';