% Created by Alessio Del Bue and Lourdes Agapito
% Last Modified: 27/04/2003
%
% License: GPLv2
%
%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; version 2, June 1991 
%
% Function: [Li, Ri] = fact_LRdecomposition(M)
% The function is called by fact_nonrigid.m and it extracts the configuration
% parameters L and the rotation matrices R from the motion matrix M. the
% solution exploits the assumption that the rotation matrix is composed by
% orthogonal rows with equal norm.
%
% Input
%
% M: Motion Matrix of size 2Fx3k
%
% Output
%
% Li: Configuration weights of size Fxk
% Ri: Rotation matrix of size 2Fx3

function [Li, Ri] = fact_LRdecomposition(M)


k = size(M,2)/3; % no. of basis-shapes
f = size(M,1)/2; % no. of frames

for i = 0:f-1
    
    Mf(1,:) = M(2*i+1,:);   % extract rowwise a 2 by 3*k submatrix from Mhat such that Mf = [l1*R1...lk*R1]
    Mf(2,:) = M(2*i+2,:);   % vectorize Mf such that [l1*r1,l1*r4,l1*r2,l1*r5...lk*r1,lk*r4,lk*r2,lk*r5,lk*r3,lk*r6]' 
%     vecMf = fact_vec(Mf);
    vecMf = Mf(:);
    
    for m=0:k-1 %gets complicated, try to decode Matt's notation
        vec6Mf(1:6,m+1) =  vecMf(6*m+1:6*m+6);  % create a 6 by k matrix from the previous vector
    end
    
    c = ones(k,1);
    Mf1 = vec6Mf*c; % computes a 6-vector 
    Mnew = [Mf1(1:2), Mf1(3:4), Mf1(5:6)]; % the 2 by 3 matrix M is structured as the following:
                                        % 1st row: [r1(l1+l2+...+lk) r2(l1+l2+...+lk) r3(l1+l2+...+lk)]
                                        % 2nd row: [r4(l1+l2+...+lk)
                                        % r5(l1+l2+...+lk) r6(l1+l2+...+lk)]    

    R = sqrtm(Mnew*Mnew')\Mnew;  
    % vecR = [R(1:2,1);R(1:2,2);R(1:2,3)]; % R matrix vectorized
    Ri(2*i+1,:) = R(1,:); % plug the F rotation matrices in a sinlgle 2*F by 3 matrix
    Ri(2*i+2,:) = R(2,:);
    for m=0:k-1 % vectorize the Mf in a slight different way from before: [l1*r1,l1*r2,l1*r3,l1*r4,l1*r5,l1*r6...
        vec6Mf(1:3,m+1) = Mf(1,3*m+1:3*(m+1))';
        vec6Mf(4:6,m+1) = Mf(2,3*m+1:3*(m+1))';        
    end    
    L = ([R(1,:),R(2,:)]*vec6Mf)'/2;
    Li(:,i+1) = L; % plug the F configuration weights in a sinlgle F by k matrix
end